@@ -1,6 +1,4 @@
 #!/usr/bin/env python
-# $Id: NameMapper.py,v 1.30 2007/04/03 01:58:20 tavis_rudd Exp $
-
 """This module supports Cheetah's optional NameMapper syntax.
 
 Overview
@@ -24,9 +22,9 @@ use information from your system on the client's website --AND-- they want to
 understand the display code and so they can maintian it themselves.
 
 You write a UI class with a 'customers' method that returns a dictionary of all
-the customer objects.  Each customer object has an 'address' method that returns
-the a dictionary with information about the customer's address.  The designers
-want to be able to access that information.
+the customer objects.  Each customer object has an 'address' method
+that returns the a dictionary with information about the customer's address.
+The designers want to be able to access that information.
 
 Using PSP, the display code for the website would look something like the
 following, assuming your servlet subclasses the class you created for managing
@@ -37,25 +35,26 @@ customer information:
 Using Cheetah's NameMapper syntax it could be any of the following:
 
    $self.customers()[$ID].address()['city']       (39 chars)
-   --OR--                                         
-   $customers()[$ID].address()['city']           
-   --OR--                                         
-   $customers()[$ID].address().city              
-   --OR--                                         
-   $customers()[$ID].address.city                
-   --OR--                                         
+   --OR--
+   $customers()[$ID].address()['city']
+   --OR--
+   $customers()[$ID].address().city
+   --OR--
    $customers()[$ID].address.city
    --OR--
-   $customers[$ID].address.city                   (27 chars)                   
-   
-   
+   $customers()[$ID].address.city
+   --OR--
+   $customers[$ID].address.city                   (27 chars)
+
+
 Which of these would you prefer to explain to the designers, who have no
 programming experience?  The last form is 15 characters shorter than the PSP
 and, conceptually, is far more accessible. With PHP or ASP, the code would be
 even messier than the PSP
 
-This is a rather extreme example and, of course, you could also just implement
-'$getCustomer($ID).city' and obey the Law of Demeter (search Google for more on that).
+This is a rather extreme example and, of course,
+you could also just implement '$getCustomer($ID).city'
+and obey the Law of Demeter (search Google for more on that).
 But good object orientated design isn't the point here.
 
 Details
@@ -65,40 +64,49 @@ The parenthesized letters below correspond to the aims in the second paragraph.
 DICTIONARY ACCESS (a)
 ---------------------
 
-NameMapper allows access to items in a dictionary using the same dotted notation
-used to access object attributes in Python.  This aspect of NameMapper is known
-as 'Unified Dotted Notation'.
+NameMapper allows access to items in a dictionary
+using the same dotted notation used to access object attributes in Python.
+This aspect of NameMapper is known as 'Unified Dotted Notation'.
+
+For example, with Cheetah it is possible to write::
+
+    $customers()['kerr'].address()  --OR--  $customers().kerr.address()
 
-For example, with Cheetah it is possible to write:
-   $customers()['kerr'].address()  --OR--  $customers().kerr.address()
 where the second form is in NameMapper syntax.
 
-This only works with dictionary keys that are also valid python identifiers:
-  regex = '[a-zA-Z_][a-zA-Z_0-9]*'
+This only works with dictionary keys that are also valid python identifiers::
+
+    regex = '[a-zA-Z_][a-zA-Z_0-9]*'
 
 
 AUTOCALLING (b,d)
 -----------------
 
-NameMapper automatically detects functions and methods in Cheetah $vars and calls
-them if the parentheses have been left off.  
+NameMapper automatically detects functions and methods in Cheetah $vars
+and calls them if the parentheses have been left off.
+
+For example if 'a' is an object, 'b' is a method::
 
-For example if 'a' is an object, 'b' is a method
   $a.b
-is equivalent to
+
+is equivalent to::
+
   $a.b()
 
-If b returns a dictionary, then following variations are possible
+If b returns a dictionary, then following variations are possible::
+
   $a.b.c  --OR--  $a.b().c  --OR--  $a.b()['c']
+
 where 'c' is a key in the dictionary that a.b() returns.
 
 Further notes:
-* NameMapper autocalls the function or method without any arguments.  Thus
-autocalling can only be used with functions or methods that either have no
-arguments or have default values for all arguments.
 
-* NameMapper only autocalls functions and methods.  Classes and callable object instances
-will not be autocalled.  
+* NameMapper autocalls the function or method without any arguments.  Thus
+  autocalling can only be used with functions or methods that either have no
+  arguments or have default values for all arguments.
+
+* NameMapper only autocalls functions and methods.
+  Classes and callable object instances will not be autocalled.
 
 * Autocalling can be disabled using Cheetah's 'useAutocalling' setting.
 
@@ -110,7 +118,7 @@ without needing to include 'self' in the variable names.  See the NAMESPACE
 CASCADING section below for details.
 
 NAMESPACE CASCADING (d)
---------------------
+-----------------------
 ...
 
 Implementation details
@@ -130,23 +138,12 @@ difference in realistic usage scenarios.
 
 Cheetah uses the optimized C version (_namemapper.c) if it has
 been compiled or falls back to the Python version if not.
-
-Meta-Data
-================================================================================
-Authors: Tavis Rudd <tavis@damnsimple.com>,
-         Chuck Esterbrook <echuck@mindspring.com>
-Version: $Revision: 1.30 $
-Start Date: 2001/04/03
-Last Revision Date: $Date: 2007/04/03 01:58:20 $
 """
-from __future__ import generators
-__author__ = "Tavis Rudd <tavis@damnsimple.com>," +\
-             "\nChuck Esterbrook <echuck@mindspring.com>"
-__revision__ = "$Revision: 1.30 $"[11:-2]
-import types
-from types import StringType, InstanceType, ClassType, TypeType
-from pprint import pformat
+
+from collections import Mapping
 import inspect
+from pprint import pformat
+from Cheetah.compat import PY2
 
 _INCLUDE_NAMESPACE_REPR_IN_NOTFOUND_EXCEPTIONS = False
 _ALLOW_WRAPPING_OF_NOTFOUND_EXCEPTIONS = True
@@ -159,81 +156,85 @@ __all__ = ['NotFound',
            'valueFromFrame',
            ]
 
+if PY2 and not hasattr(inspect.imp, 'get_suffixes'):
+    # This is to fix broken behavior of the inspect module under the
+    # Google App Engine
+    setattr(inspect.imp, 'get_suffixes', lambda: [('.py', 'U', 1)])
 
-## N.B. An attempt is made at the end of this module to import C versions of
-## these functions.  If _namemapper.c has been compiled succesfully and the
-## import goes smoothly, the Python versions defined here will be replaced with
-## the C versions.
+# N.B. An attempt is made at the end of this module to import C versions of
+# these functions.  If _namemapper.c has been compiled succesfully and the
+# import goes smoothly, the Python versions defined here will be replaced with
+# the C versions.
 
-class NotFound(LookupError):
-    pass
 
 def _raiseNotFoundException(key, namespace):
-    excString = "cannot find '%s'"%key
+    excString = "cannot find '%s'" % key
     if _INCLUDE_NAMESPACE_REPR_IN_NOTFOUND_EXCEPTIONS:
-        excString += ' in the namespace %s'%pformat(namespace)
+        excString += ' in the namespace %s' % pformat(namespace)
     raise NotFound(excString)
 
+
 def _wrapNotFoundException(exc, fullName, namespace):
     if not _ALLOW_WRAPPING_OF_NOTFOUND_EXCEPTIONS:
-        raise 
+        raise
     else:
         excStr = exc.args[0]
-        if excStr.find('while searching')==-1: # only wrap once!
-            excStr +=" while searching for '%s'"%fullName
+        if excStr.find('while searching') == -1:  # only wrap once!
+            excStr += " while searching for '%s'" % fullName
             if _INCLUDE_NAMESPACE_REPR_IN_NOTFOUND_EXCEPTIONS:
-                excStr += ' in the namespace %s'%pformat(namespace)
+                excStr += ' in the namespace %s' % pformat(namespace)
             exc.args = (excStr,)
         raise
-    
+
+
+def _isInstanceOrClass(obj):
+    if isinstance(obj, type):
+        # oldstyle
+        return True
+
+    if hasattr(obj, "__class__"):
+        # newstyle
+        if hasattr(obj, 'mro'):
+            # type/class
+            return True
+        elif (hasattr(obj, '__func__') or hasattr(obj, '__code__') or
+                hasattr(obj, '__self__')):
+            # method, func, or builtin func
+            return False
+        elif hasattr(obj, '__init__'):
+            # instance
+            return True
+    return False
+
+
 def hasKey(obj, key):
     """Determine if 'obj' has 'key' """
-    if hasattr(obj,'has_key') and obj.has_key(key):
+    if isinstance(obj, Mapping) and key in obj:
         return True
     elif hasattr(obj, key):
         return True
     else:
         return False
 
-def valueForKey(obj, key):
-    if hasattr(obj, 'has_key') and obj.has_key(key):
-        return obj[key]
-    elif hasattr(obj, key):
-        return getattr(obj, key)
-    else:
-        _raiseNotFoundException(key, obj)
 
 def _valueForName(obj, name, executeCallables=False):
-    nameChunks=name.split('.')
-    for i in range(len(nameChunks)):
-        key = nameChunks[i]
-        if hasattr(obj, 'has_key') and obj.has_key(key):
+    nameChunks = name.split('.')
+    for key in nameChunks:
+        if isinstance(obj, Mapping) and key in obj:
             nextObj = obj[key]
-        elif hasattr(obj, key):
-            nextObj = getattr(obj, key)
         else:
-            _raiseNotFoundException(key, obj)
-        
-        if (executeCallables and callable(nextObj)
-            and (type(nextObj) not in (InstanceType, ClassType))):
+            try:
+                nextObj = getattr(obj, key)
+            except AttributeError:
+                _raiseNotFoundException(key, obj)
+
+        if executeCallables and hasattr(nextObj, '__call__') and \
+                not _isInstanceOrClass(nextObj):
             obj = nextObj()
         else:
             obj = nextObj
     return obj
 
-def valueForName(obj, name, executeCallables=False):
-    try:
-        return _valueForName(obj, name, executeCallables)
-    except NotFound, e:
-        _wrapNotFoundException(e, fullName=name, namespace=obj)
-
-def valueFromSearchList(searchList, name, executeCallables=False):
-    key = name.split('.')[0]
-    for namespace in searchList:
-        if hasKey(namespace, key):
-            return _valueForName(namespace, name,
-                                executeCallables=executeCallables)
-    _raiseNotFoundException(key, searchList)
 
 def _namespaces(callerFrame, searchList=None):
     yield callerFrame.f_locals
@@ -243,39 +244,9 @@ def _namespaces(callerFrame, searchList=None):
     yield callerFrame.f_globals
     yield __builtins__
 
-def valueFromFrameOrSearchList(searchList, name, executeCallables=False,
-                               frame=None):
-    def __valueForName():
-        try:
-            return _valueForName(namespace, name, executeCallables=executeCallables)
-        except NotFound, e:
-            _wrapNotFoundException(e, fullName=name, namespace=searchList)
-    try:
-        if not frame:
-            frame = inspect.stack()[1][0]
-        key = name.split('.')[0]
-        for namespace in _namespaces(frame, searchList):
-            if hasKey(namespace, key): return __valueForName()
-        _raiseNotFoundException(key, searchList)
-    finally:
-        del frame
-
-def valueFromFrame(name, executeCallables=False, frame=None):
-    # @@TR consider implementing the C version the same way
-    # at the moment it provides a seperate but mirror implementation
-    # to valueFromFrameOrSearchList
-    try: 
-        if not frame:
-            frame = inspect.stack()[1][0]
-        return valueFromFrameOrSearchList(searchList=None,
-                                          name=name,
-                                          executeCallables=executeCallables,
-                                          frame=frame)
-    finally:
-        del frame
 
 def hasName(obj, name):
-    #Not in the C version
+    # Not in the C version
     """Determine if 'obj' has the 'name' """
     key = name.split('.')[0]
     if not hasKey(obj, key):
@@ -285,16 +256,78 @@ def hasName(obj, name):
         return True
     except NotFound:
         return False
+
+
 try:
-    from _namemapper import NotFound, valueForKey, valueForName, \
+    from Cheetah._namemapper import NotFound, valueForKey, valueForName, \
          valueFromSearchList, valueFromFrameOrSearchList, valueFromFrame
-    # it is possible with Jython or Windows, for example, that _namemapper.c hasn't been compiled
     C_VERSION = True
-except:
+except Exception:
+    class NotFound(LookupError):
+        pass
+
+    def valueForKey(obj, key):
+        if isinstance(obj, Mapping) and key in obj:
+            return obj[key]
+        elif hasattr(obj, key):
+            return getattr(obj, key)
+        else:
+            _raiseNotFoundException(key, obj)
+
+    def valueForName(obj, name, executeCallables=False):
+        try:
+            return _valueForName(obj, name, executeCallables)
+        except NotFound as e:
+            _wrapNotFoundException(e, fullName=name, namespace=obj)
+
+    def valueFromSearchList(searchList, name, executeCallables=False):
+        key = name.split('.')[0]
+        for namespace in searchList:
+            if hasKey(namespace, key):
+                return _valueForName(namespace, name,
+                                     executeCallables=executeCallables)
+        _raiseNotFoundException(key, searchList)
+
+    def valueFromFrame(name, executeCallables=False, frame=None):
+        # @@TR consider implementing the C version the same way
+        # at the moment it provides a seperate but mirror implementation
+        # to valueFromFrameOrSearchList
+        try:
+            if not frame:
+                frame = inspect.stack()[1][0]
+            return valueFromFrameOrSearchList(
+                searchList=None, name=name,
+                executeCallables=executeCallables, frame=frame)
+        finally:
+            del frame
+
+    def valueFromFrameOrSearchList(searchList, name, executeCallables=False,
+                                   frame=None):
+        def __valueForName():
+            try:
+                return _valueForName(namespace, name,
+                                     executeCallables=executeCallables)
+            except NotFound as e:
+                _wrapNotFoundException(e, fullName=name, namespace=searchList)
+        try:
+            if not frame:
+                frame = inspect.stack()[1][0]
+            key = name.split('.')[0]
+            for namespace in _namespaces(frame, searchList):
+                if hasKey(namespace, key):
+                    return __valueForName()
+            _raiseNotFoundException(key, searchList)
+        finally:
+            del frame
+
+    # It is possible with PyPy, Jython or Windows, for example,
+    # that _namemapper.c hasn't been compiled.
     C_VERSION = False
 
+
 ##################################################
-## CLASSES
+# CLASSES
+
 
 class Mixin:
     """@@ document me"""
@@ -305,16 +338,18 @@ class Mixin:
         return valueForKey(self, key)
 
 ##################################################
-## if run from the command line ##
+# if run from the command line ##
+
 
 def example():
     class A(Mixin):
         classVar = 'classVar val'
-        def method(self,arg='method 1 default arg'):
+
+        def method(self, arg='method 1 default arg'):
             return arg
 
         def method2(self, arg='meth 2 default arg'):
-            return {'item1':arg}
+            return {'item1': arg}
 
         def method3(self, arg='meth 3 default'):
             return arg
@@ -324,6 +359,7 @@ def example():
 
     a = A()
     a.one = 'valueForOne'
+
     def function(whichOne='default'):
         values = {
             'default': 'default output',
@@ -336,20 +372,18 @@ def example():
         'func': function,
         'method': a.method3,
         'item': 'itemval',
-        'subDict': {'nestedMethod':a.method3}
+        'subDict': {'nestedMethod': a.method3}
         }
-    b = 'this is local b'
+    # b = 'this is local b'
+
+    print(valueForKey(a.dic, 'subDict'))
+    print(valueForName(a, 'dic.item'))
+    print(valueForName(vars(), 'b'))
+    print(valueForName(__builtins__, 'dir')())
+    print(valueForName(vars(), 'a.classVar'))
+    print(valueForName(vars(), 'a.dic.func', executeCallables=True))
+    print(valueForName(vars(), 'a.method2.item1', executeCallables=True))
 
-    print valueForKey(a.dic,'subDict')
-    print valueForName(a, 'dic.item')
-    print valueForName(vars(), 'b')
-    print valueForName(__builtins__, 'dir')()
-    print valueForName(vars(), 'a.classVar')
-    print valueForName(vars(), 'a.dic.func', executeCallables=True)
-    print valueForName(vars(), 'a.method2.item1', executeCallables=True)
 
 if __name__ == '__main__':
     example()
-
-
-
