@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-'''
+"""
 Core module of Cheetah's Unit-testing framework
 
 TODO
@@ -7,18 +7,20 @@ TODO
 # combo tests
 # negative test cases for expected exceptions
 # black-box vs clear-box testing
-# do some tests that run the Template for long enough to check that the refresh code works
-'''
+# do some tests that run the Template for long enough to check
+# that the refresh code works
+"""
 
 import sys
 import unittest
 
 from Cheetah.Tests import SyntaxAndOutput
 from Cheetah.Tests import NameMapper
+from Cheetah.Tests import NameMapper_pure
 from Cheetah.Tests import Misc
 from Cheetah.Tests import Filters
 from Cheetah.Tests import Template
-from Cheetah.Tests import Cheps
+# from Cheetah.Tests import Cheps
 from Cheetah.Tests import Parser
 from Cheetah.Tests import Regressions
 from Cheetah.Tests import Unicode
@@ -30,9 +32,10 @@ SyntaxAndOutput.install_eols()
 suites = [
    unittest.findTestCases(SyntaxAndOutput),
    unittest.findTestCases(NameMapper),
+   unittest.findTestCases(NameMapper_pure),
    unittest.findTestCases(Filters),
    unittest.findTestCases(Template),
-   #unittest.findTestCases(Cheps),
+   # unittest.findTestCases(Cheps),
    unittest.findTestCases(Regressions),
    unittest.findTestCases(Unicode),
    unittest.findTestCases(Misc),
@@ -44,10 +47,14 @@ if not sys.platform.startswith('java'):
     suites.append(unittest.findTestCases(CheetahWrapper))
 
 if __name__ == '__main__':
-    runner = unittest.TextTestRunner()
     if 'xml' in sys.argv:
-        import xmlrunner
+        from Cheetah.Tests import xmlrunner
         runner = xmlrunner.XMLTestRunner(filename='Cheetah-Tests.xml')
-    
-    results = runner.run(unittest.TestSuite(suites))
+    else:
+        runner = unittest.TextTestRunner()
 
+    results = runner.run(unittest.TestSuite(suites))
+    if results.wasSuccessful():
+        sys.exit(0)
+    else:
+        sys.exit(1)
